/*==========================================================
 * Program : gsmdevices.c                  Project : smslink
 * Author  : Philippe Andersson.
 * Date    : 21/06/00
 * Version : 0.04b
 * Notice  : Shamelessly copied from Riccardo Facchetti's modems.c
 *           in libmodem-1.0.0 (c) 1997 Riccardo Facchetti under GNU GPL.
 *           (c) Les Ateliers du Heron, 1998 for Scitex Europe, S.A.
 * Comment : Handling routines for /etc/gsmdevices database.
 *
 * Modification History :
 * - 0.01a (13/08/98) : Initial release.
 * ++++ Switched to Beta ++++
 * - 0.02b (29/03/00) : Modified /etc/gsmdevices record structure
 *   to hold the default sending mode. Adapted demangle_gsmdev_entry()
 *   accordingly.
 * - 0.03b (20/05/00) : Added translatemodname() function to
 *   handle to pseudo-driver file, and adapted demangle_gsmdev_entry()
 *   accordingly. Cosmetics.
 * - 0.04b (21/06/00) : Adapted translatemodname() and
 *   demangle_gsmdev_entry() to handle the new capmatrix structure.
 *========================================================*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>

#include <dial/modems.h>
#include <dial/mdmerrno.h>
#include <termios.h>

#include "sms_serv.h"
#include "gsmdev.h"

#define BUFSIZE		128

/*========================================================*/
/* For debugging purposes only - comment out for normal compile */
/* #define INCL_DEBUG_CODE */

/*========================================================*/
/**********           GLOBAL VARIABLES             ********/
/*========================================================*/

static int gsmdeventry = 0;
static FILE *gsmdevfp;

/*========================================================*/
/**********               FUNCTIONS                ********/
/*========================================================*/
int translatemodname (char *model, char *driverfilename, struct gsms_def *mdm)
{
  FILE *driverfile;
  int found;
  int l;
  char *line;
  char *token;
  
  /*---------------------------------------Initialization */
  line = (char *) malloc ((BUFFSIZE + 1) * sizeof (char));
  if (! line) {
    fprintf (stderr, "sms_serv: in translatemodname(): can't malloc()\n");
    return (-1);
  }
  line[0] = '\0';
  found = FALSE;
  /*-------------------------------------Open driver file */
  if ((driverfile = fopen (driverfilename, "r")) == NULL) {
    fprintf (stderr, "sms_serv: in translatemodname(): can't fopen() %s\n",
            driverfilename);
    return (-1);
  }
  /*--------------------------------------Loop and search */
  while ((! found) && (fgets (line, BUFFSIZE, driverfile) != NULL)) {
    if ((line[0] != '#') && (line[0] != '\n') && (line[0] != '\0')){
      /* line is not a comment or a blank */
      if ((token = strtok (line, ":")) == NULL) {
        return (-1);
      }
      if (strcmp (token, model) == 0) {
        /* we've found it - let's extract the cap. matrix items */
        /*.......................................capflags */
	if ((token = strtok (NULL, ":")) == NULL) {
          return (-1);
        }
	mdm->capmatrix.capflags = atoi (token);
#ifdef INCL_DEBUG_CODE
        fprintf (stderr, "GSM Model %s has cap. flags : [%d]\n", model,
	        mdm->capmatrix.capflags);
#endif
        /*....................................date format */
	if ((token = strtok (NULL, ":")) == NULL) {
          return (-1);
        }
	l = strlen (token);
	if (l) {
	  if (l > 3) {
	    strncpy (mdm->capmatrix.datefmt, token, l);
	  }
	  else {
	    strcpy (mdm->capmatrix.datefmt, token);
	  }
	}
	else {
	  /* empty field - let's default to 'dmy' */
	  strcpy (mdm->capmatrix.datefmt, "dmy");
	}
#ifdef INCL_DEBUG_CODE
        fprintf (stderr, "GSM Model %s has date format : [%s]\n", model,
	        mdm->capmatrix.datefmt);
#endif
	/*................................................*/
	found = TRUE;
      }
    }                              /* if (line[0] != '#') */
  }                                    /* while (fgets... */
  /*------------------------------------------Conclusions */
  fclose (driverfile);
  free (line);
  return (0);
}                                  /* translatemodname () */
/*========================================================*/
static int open_gsmdevs (void)
/*
 * open/close_gsmdevs: open and close the /etc/gsmdevices file
 */
{
    if ((gsmdevfp = fopen (GSMDEVFILE, "r")) == NULL)
        return FAILURE;
    return SUCCESS;
}                                      /* open_gsmdevs () */
/*========================================================*/
static void close_gsmdevs (void)
{
    fclose (gsmdevfp);
}                                     /* close_gsmdevs () */
/*========================================================*/
struct gsms_def *getgsmdevbynam (char *line)
/*
 * getgsmdevbynam: returns a pointer to the gsmdev entry named
 *            (char *) line, NULL if line is not present in
 *            /etc/gsmdevices.
 */
{
    static struct gsms_def mdm;
    char *buffer;

    buffer = mdmalloc(BUFSIZE);

    if (!buffer) {
	mdmerrno = -EMDMEM;
	return NULL;
    }

    mdmerrno = 0;

    if (open_gsmdevs() == FAILURE) {
	mdmfree (buffer);
        mdmerrno = -ENOMDMFILE;
        return NULL;
    }

    while (fgets (buffer, BUFSIZE, gsmdevfp) != NULL) {
        buffer[strlen (buffer) - 1] = '\0';

        if (buffer[0] == '#' || buffer[0] == '\0' || buffer[0] == '\n')
            continue;

        if (demangle_gsmdev_entry (&mdm, buffer) == FAILURE) {
            mdmerrno = -EMDMBADCONF;
            continue;
        }
        if (!strcmp (mdm.device, line)) {
            close_gsmdevs ();
	    mdmfree (buffer);
            return (&mdm);
        }
    }
    close_gsmdevs ();
    mdmerrno = -ENOMDMLINE;
    return NULL;
}                                    /* getgsmdevbynam () */
/*========================================================*/
int getgsmdevscount (int validation)
/*
 * getgsmdevscount: count the number of valid (?) entries in
 *            /etc/gsmdevices.
 */
{
    static struct gsms_def mdm;
    char *buffer;
    int nvalidentries = 0;

    buffer = mdmalloc(BUFSIZE);

    if (!buffer) {
	mdmerrno = -EMDMEM;
	return (0);
    }

    mdmerrno = 0;

    if (open_gsmdevs() == FAILURE) {
	mdmfree (buffer);
        mdmerrno = -ENOMDMFILE;
        return (0);
    }

    while (fgets (buffer, BUFSIZE, gsmdevfp) != NULL) {
        buffer[strlen (buffer) - 1] = '\0';

        if (buffer[0] == '#' || buffer[0] == '\0' || buffer[0] == '\n')
            continue;

        if (validation && (demangle_gsmdev_entry (&mdm, buffer) == FAILURE)) {
            mdmerrno = -EMDMBADCONF;
            continue;
        }
	nvalidentries++;
    }
    close_gsmdevs ();
    return (nvalidentries);
}                                   /* getgsmdevscount () */
/*========================================================*/
void setgsmdevs (void)
/*
 * setgsmdevs: set the library for modem sequential search from the first to the
 *          end. See getnextgsmdev ().
 */
{
    gsmdeventry = 0;
}                                        /* setgsmdevs () */
/*========================================================*/
struct gsms_def *getnextgsmdev (void)
/*
 * getnextgsmdev: get the next modem entry in /etc/gsmdevices
 *             file returns NULL if we are over the last entry.
 */
{
    static struct gsms_def mdm;
    int i = 0;
    char *buffer;

    buffer = mdmalloc (BUFSIZE);

    if (!buffer) {
	mdmerrno = -EMDMEM;
	return NULL;
    }


    mdmerrno = 0;

    if (open_gsmdevs () == FAILURE) {
	mdmfree (buffer);
        mdmerrno = -ENOMDMFILE;
        return NULL;
    }

    while (fgets (buffer, BUFSIZE, gsmdevfp) != NULL) {
        buffer[strlen (buffer) - 1] = '\0';

        if (buffer[0] == '#' || buffer[0] == '\0' || buffer[0] == '\n')
            continue;

        if (i == gsmdeventry) {
            if (demangle_gsmdev_entry (&mdm, buffer) == FAILURE) {
                mdmerrno = -EMDMBADCONF;
                continue;
            }
            gsmdeventry++;
            close_gsmdevs ();
	    mdmfree (buffer);
            return (&mdm);
        }
        i++;
    }
    close_gsmdevs ();
    mdmfree(buffer);
    return NULL;
}                                     /* getnextgsmdev () */
/*========================================================*/
static int demangle_gsmdev_entry (struct gsms_def *mdm, char *mdmstr)
/*
 * demangle_gsmdev_entry: get a gsmdev pointer that point to an existing
 * gsm_def structure and a string to demangle. Returns FAILURE or SUCCESS.
 * The buffer pointed to by mdmstr is modified in the process.
 */
{
    char *ptr, *pptr, *errptr;
    char model[MAXMODLEN + 1];

    /*....................................mdm->free (int) */
    /* is left uninitialized - dynamic data */
    
    /*...............................mdm->device (char *) */
    if ((ptr = strchr (mdmstr, ':')) == NULL)
        return FAILURE;
    *ptr = '\0';
    if ((strlen (mdmstr) == 0) || (strlen (mdmstr) > MAXDEVLEN))
        return (FAILURE);
    strcpy (mdm->device, mdmstr);
    ptr++;

    /*..................................mdm->PIN (char *) */
    if (*ptr == ':') {
        /* empty field */
        mdm->PIN[0] = '\0';
        *ptr = '\0';
        ptr++;
    }
    else {
        if ((pptr = strchr (ptr, ':')) == NULL)
            return FAILURE;
        *pptr = '\0';
        if (strlen (ptr) > PINLEN)
            return (FAILURE);
	strcpy (mdm->PIN, ptr);
        ptr = pptr + 1;
    }

    /*..................................mdm->PUK (char *) */
    if (*ptr == ':') {
        /* empty field */
        mdm->PUK[0] = '\0';
        *ptr = '\0';
        ptr++;
    }
    else {
        if ((pptr = strchr (ptr, ':')) == NULL)
            return FAILURE;
        *pptr = '\0';
        if (strlen (ptr) > PUKLEN)
            return (FAILURE);
        strcpy (mdm->PUK, ptr);
        ptr = pptr + 1;
    }

    /*.................................mdm->addr (char *) */
    if (*ptr == ':') {
        /* empty field */
        mdm->addr[0] = '\0';
        *ptr = '\0';
        ptr++;
    }
    else {
        if ((pptr = strchr (ptr, ':')) == NULL)
            return FAILURE;
        *pptr = '\0';
        if (strlen (ptr) > MAXPHNUMLEN)
            return (FAILURE);
        strcpy (mdm->addr, ptr);
        ptr = pptr + 1;
    }

    /*...............................mdm->defsca (char *) */
    if (*ptr == ':') {
        /* empty field */
        mdm->defsca[0] = '\0';
        *ptr = '\0';
        ptr++;
    }
    else {
        if ((pptr = strchr (ptr, ':')) == NULL)
            return FAILURE;
        *pptr = '\0';
        if (strlen (ptr) > MAXPHNUMLEN)
            return (FAILURE);
        strcpy (mdm->defsca, ptr);
        ptr = pptr + 1;
    }

    /*.................................mdm->defmode (int) */
    if (*ptr == ':') {
        /* empty field */
        mdm->defmode = SMS_MODE_DEFAULT;
        *ptr = '\0';
        ptr++;
    }
    else {
        if ((pptr = strchr (ptr, ':')) == NULL)
            return FAILURE;
        *pptr = '\0';
        if (strlen (ptr) == 0)
            return (FAILURE);
        mdm->defmode = strtol (ptr, &errptr, 0);
	if (*errptr != '\0')
            return (FAILURE);
        ptr = pptr + 1;
    }

    /*.....................................mdm->capmatrix */
    if (*ptr == ':') {
        /* empty field */
        mdm->capmatrix.capflags = 0;
	mdm->capmatrix.datefmt[0] = '\0';
        *ptr = '\0';
        ptr++;
    }
    else {
        if ((pptr = strchr (ptr, ':')) == NULL)
            return FAILURE;
        *pptr = '\0';
        if (strlen (ptr) > MAXMODLEN)
            return (FAILURE);
        strcpy (model, ptr);
	/* Now translate model into capacity matrix */
	if ((translatemodname (model, GSMCAPS_FILE, mdm)) == -1) {
	  return (FAILURE);
	}
        ptr = pptr + 1;
    }

    /*.............................mdm->provider (char *) */
    /*                                 !!! Last field !!! */
    if (*ptr == '\n' || *ptr == '\0' || *ptr == ':') {
        mdm->provider[0] = '\0';
        return SUCCESS;
    }
    else {
        if (strlen (ptr) > MAXDEVLEN)
            return (FAILURE);
        strcpy (mdm->provider, ptr);
#ifdef INCL_DEBUG_CODE
        fprintf (stderr, "Provider : [%s]\n", mdm->provider);
#endif
    }

    return SUCCESS;
}                             /* demangle_gsmdev_entry () */
/*==========================================================
 * EOF : gsmdevices.c
 *===================*/
